@@ -107,7 +107,6 @@ struct _GstCollectPadsPrivate
 {
   /* with LOCK and/or STREAM_LOCK */
   gboolean started;
-  gboolean stream_started;
 
   /* with STREAM_LOCK */
   guint32 cookie;               /* @data list cookie */
@@ -234,7 +233,6 @@ gst_collect_pads_init (GstCollectPads * pads)
   pads->priv->queuedpads = 0;
   pads->priv->eospads = 0;
   pads->priv->started = FALSE;
-  pads->priv->stream_started = FALSE;
 
   g_rec_mutex_init (&pads->stream_lock);
 
@@ -288,7 +286,7 @@ gst_collect_pads_finalize (GObject * object)
 /**
  * gst_collect_pads_new:
  *
- * Create a new instance of #GstCollectsPads.
+ * Create a new instance of #GstCollectPads.
  *
  * MT safe.
  *
@@ -365,7 +363,7 @@ gst_collect_pads_set_compare_function (GstCollectPads * pads,
 
 /**
  * gst_collect_pads_set_function:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @func: the function to set
  * @user_data: user data passed to the function
  *
@@ -425,7 +423,7 @@ unref_data (GstCollectData * data)
 
 /**
  * gst_collect_pads_set_event_function:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @func: the function to set
  * @user_data: user data passed to the function
  *
@@ -453,7 +451,7 @@ gst_collect_pads_set_event_function (GstCollectPads * pads,
 
 /**
  * gst_collect_pads_set_query_function:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @func: the function to set
  * @user_data: user data passed to the function
  *
@@ -481,7 +479,7 @@ gst_collect_pads_set_query_function (GstCollectPads * pads,
 
 /**
 * gst_collect_pads_clip_running_time:
-* @pads: the collectspads to use
+* @pads: the collectpads to use
 * @cdata: collect data of corresponding pad
 * @buf: buffer being clipped
 * @outbuf: output buffer with running time, or NULL if clipped
@@ -521,7 +519,7 @@ gst_collect_pads_clip_running_time (GstCollectPads * pads,
 
  /**
  * gst_collect_pads_set_clip_function:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @clipfunc: clip function to install
  * @user_data: user data to pass to @clip_func
  *
@@ -541,7 +539,7 @@ gst_collect_pads_set_clip_function (GstCollectPads * pads,
 
 /**
  * gst_collect_pads_add_pad:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @pad: (transfer none): the pad to add
  * @size: the size of the returned #GstCollectData structure
  * @destroy_notify: function to be called before the returned #GstCollectData
@@ -638,7 +636,7 @@ find_pad (GstCollectData * data, GstPad * pad)
 
 /**
  * gst_collect_pads_remove_pad:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @pad: (transfer none): the pad to remove
  *
  * Remove a pad from the collection of collect pads. This function will also
@@ -757,7 +755,7 @@ gst_collect_pads_set_flushing_unlocked (GstCollectPads * pads,
 
 /**
  * gst_collect_pads_set_flushing:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @flushing: desired state of the pads
  *
  * Change the flushing state of all the pads in the collection. No pad
@@ -782,7 +780,7 @@ gst_collect_pads_set_flushing (GstCollectPads * pads, gboolean flushing)
 
 /**
  * gst_collect_pads_start:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  *
  * Starts the processing of data in the collect_pads.
  *
@@ -823,7 +821,7 @@ gst_collect_pads_start (GstCollectPads * pads)
 
 /**
  * gst_collect_pads_stop:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  *
  * Stops the processing of data in the collect_pads. this function
  * will also unblock any blocking operations.
@@ -871,7 +869,6 @@ gst_collect_pads_stop (GstCollectPads * pads)
     unref_data (pads->priv->earliest_data);
   pads->priv->earliest_data = NULL;
   pads->priv->earliest_time = GST_CLOCK_TIME_NONE;
-  pads->priv->stream_started = FALSE;
 
   GST_OBJECT_UNLOCK (pads);
   /* Wake them up so they can end the chain functions. */
@@ -882,7 +879,7 @@ gst_collect_pads_stop (GstCollectPads * pads)
 
 /**
  * gst_collect_pads_peek:
- * @pads: the collectspads to peek
+ * @pads: the collectpads to peek
  * @data: the data to use
  *
  * Peek at the buffer currently queued in @data. This function
@@ -914,7 +911,7 @@ gst_collect_pads_peek (GstCollectPads * pads, GstCollectData * data)
 
 /**
  * gst_collect_pads_pop:
- * @pads: the collectspads to pop
+ * @pads: the collectpads to pop
  * @data: the data to use
  *
  * Pop the buffer currently queued in @data. This function
@@ -964,7 +961,7 @@ gst_collect_pads_clear (GstCollectPads * pads, GstCollectData * data)
 
 /**
  * gst_collect_pads_available:
- * @pads: the collectspads to query
+ * @pads: the collectpads to query
  *
  * Query how much bytes can be read from each queued buffer. This means
  * that the result of this call is the maximum number of bytes that can
@@ -1033,7 +1030,7 @@ not_filled:
 
 /**
  * gst_collect_pads_flush:
- * @pads: the collectspads to query
+ * @pads: the collectpads to query
  * @data: the data to use
  * @size: the number of bytes to flush
  *
@@ -1079,7 +1076,7 @@ gst_collect_pads_flush (GstCollectPads * pads, GstCollectData * data,
 
 /**
  * gst_collect_pads_read_buffer:
- * @pads: the collectspads to query
+ * @pads: the collectpads to query
  * @data: the data to use
  * @size: the number of bytes to read
  *
@@ -1117,7 +1114,7 @@ gst_collect_pads_read_buffer (GstCollectPads * pads, GstCollectData * data,
 
 /**
  * gst_collect_pads_take_buffer:
- * @pads: the collectspads to query
+ * @pads: the collectpads to query
  * @data: the data to use
  * @size: the number of bytes to read
  *
@@ -1147,7 +1144,7 @@ gst_collect_pads_take_buffer (GstCollectPads * pads, GstCollectData * data,
 
 /**
  * gst_collect_pads_set_waiting:
- * @pads: the collectspads
+ * @pads: the collectpads
  * @data: the data to use
  * @waiting: boolean indicating whether this pad should operate
  *           in waiting or non-waiting mode
@@ -1563,7 +1560,7 @@ exit:
 
 /**
  * gst_collect_pads_event_default:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @data: collect data of corresponding pad
  * @event: event being processed
  * @discard: process but do not send event downstream
@@ -1715,14 +1712,9 @@ gst_collect_pads_event_default (GstCollectPads * pads, GstCollectData * data,
       goto eat;
     }
     case GST_EVENT_STREAM_START:
-      /* let the only the first one go through */
-      if (!pads->priv->stream_started) {
-        pads->priv->stream_started = TRUE;
-        goto forward;
-      } else {
-        goto eat;
-      }
-      break;
+      /* drop stream start events, element must create its own start event,
+       * we can't just forward the first random stream start event we get */
+      goto eat;
     case GST_EVENT_CAPS:
       goto eat;
     default:
@@ -1804,7 +1796,7 @@ pad_removed:
 
 /**
  * gst_collect_pads_query_default:
- * @pads: the collectspads to use
+ * @pads: the collectpads to use
  * @data: collect data of corresponding pad
  * @query: query being processed
  * @discard: process but do not send event downstream
